@@ -58,12 +58,36 @@ impl BrowserState {
         }))
     }
 
+    /// build a cmdResult asking for the addition of a new state
+    /// being a browser state similar to the current one but with
+    /// different options or a different root, or both
+    fn modified(
+        &self,
+        screen: Screen,
+        root: PathBuf,
+        options: TreeOptions,
+        in_new_panel: bool,
+        con: &AppContext,
+    ) -> CmdResult {
+        let tree = self.displayed_tree();
+        let mut new_state = BrowserState::new(root, options, screen, con, &Dam::unlimited());
+        if let Ok(Some(bs)) = &mut new_state {
+            if tree.selection != 0 {
+                bs.displayed_tree_mut().try_select_path(&tree.selected_line().path);
+            }
+        }
+        CmdResult::from_optional_state(
+            new_state,
+            in_new_panel,
+        )
+    }
+
     pub fn root(&self) -> &Path {
         self.tree.root()
     }
 
-    pub fn page_height(screen: Screen) -> i32 {
-        i32::from(screen.height) - 2
+    pub fn page_height(screen: Screen) -> usize {
+        screen.height as usize - 2 // br shouldn't be displayed when the screen is smaller
     }
 
     /// return a reference to the currently displayed tree, which
@@ -147,6 +171,10 @@ impl BrowserState {
 
 impl PanelState for BrowserState {
 
+    fn tree_root(&self) -> Option<&Path> {
+        Some(self.root())
+    }
+
     fn get_type(&self) -> PanelStateType {
         PanelStateType::Tree
     }
@@ -179,7 +207,7 @@ impl PanelState for BrowserState {
         let tree = self.displayed_tree();
         let mut selection = tree.selected_line().as_selection();
         selection.line = tree.options.pattern.pattern
-            .get_match_line_count(&selection.path)
+            .get_match_line_count(selection.path)
             .unwrap_or(0);
         Some(selection)
     }
@@ -201,10 +229,7 @@ impl PanelState for BrowserState {
         let tree = self.displayed_tree();
         let mut options = tree.options.clone();
         change_options(&mut options);
-        CmdResult::from_optional_state(
-            BrowserState::new(tree.root().clone(), options, screen, con, &Dam::unlimited()),
-            in_new_panel,
-        )
+        self.modified(screen, tree.root().clone(), options, in_new_panel, con)
     }
 
     fn clear_pending(&mut self) {
@@ -218,7 +243,7 @@ impl PanelState for BrowserState {
         _screen: Screen,
         _con: &AppContext,
     ) -> Result<CmdResult, ProgramError> {
-        self.displayed_tree_mut().try_select_y(y as i32);
+        self.displayed_tree_mut().try_select_y(y as usize);
         Ok(CmdResult::Keep)
     }
 
@@ -248,7 +273,13 @@ impl PanelState for BrowserState {
         if pat.is_none() {
             self.filtered_tree = None;
         }
-        self.pending_pattern = pat;
+        if let Some(filtered_tree) = &self.filtered_tree {
+            if pat != filtered_tree.options.pattern {
+                self.pending_pattern = pat;
+            }
+        } else {
+            self.pending_pattern = pat;
+        }
         Ok(CmdResult::Keep)
     }
 
@@ -342,15 +373,15 @@ impl PanelState for BrowserState {
             }
             Internal::page_down => {
                 let tree = self.displayed_tree_mut();
-                if page_height < tree.lines.len() as i32 {
-                    tree.try_scroll(page_height, page_height);
+                if !tree.try_scroll(page_height as i32, page_height) {
+                    tree.try_select_last(page_height);
                 }
                 CmdResult::Keep
             }
             Internal::page_up => {
                 let tree = self.displayed_tree_mut();
-                if page_height < tree.lines.len() as i32 {
-                    tree.try_scroll(-page_height, page_height);
+                if !tree.try_scroll(page_height as i32 * -1, page_height) {
+                    tree.try_select_first();
                 }
                 CmdResult::Keep
             }
@@ -397,7 +428,29 @@ impl PanelState for BrowserState {
             }
             Internal::parent => self.go_to_parent(screen, con, bang),
             Internal::print_tree => {
-                print::print_tree(&self.displayed_tree(), cc.app.screen, &cc.app.panel_skin, con)?
+                print::print_tree(self.displayed_tree(), cc.app.screen, cc.app.panel_skin, con)?
+            }
+            Internal::root_up => {
+                let tree = self.displayed_tree();
+                let root = tree.root();
+                if let Some(new_root) = root.parent() {
+                    self.modified(screen, new_root.to_path_buf(), tree.options.clone(), bang, con)
+                } else {
+                    CmdResult::error(format!("{:?} has no parent", root))
+                }
+            }
+            Internal::root_down => {
+                let tree = self.displayed_tree();
+                if tree.selection > 0 {
+                    let root_len = tree.root().components().count();
+                    let new_root = tree.selected_line().path
+                        .components()
+                        .take(root_len + 1)
+                        .collect();
+                    self.modified(screen, new_root, tree.options.clone(), bang, con)
+                } else {
+                    CmdResult::error("No selected line")
+                }
             }
             Internal::select_first => {
                 self.displayed_tree_mut().try_select_first();
@@ -534,8 +587,8 @@ impl PanelState for BrowserState {
         disc: &DisplayContext,
     ) -> Result<(), ProgramError> {
         let dp = DisplayableTree {
-            app_state: Some(&disc.app_state),
-            tree: &self.displayed_tree(),
+            app_state: Some(disc.app_state),
+            tree: self.displayed_tree(),
             skin: &disc.panel_skin.styles,
             ext_colors: &disc.con.ext_colors,
             area: disc.state_area.clone(),
